@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2007-2012  Red Hat, Inc.
+# Copyright 2007-2015  Red Hat, Inc.
 # This file is part of python-fedora
 #
 # python-fedora is free software; you can redistribute it and/or
@@ -22,17 +22,26 @@ This module provides a client interface for bodhi.
 
 .. moduleauthor:: Luke Macken <lmacken@redhat.com>
 .. moduleauthor:: Toshio Kuratomi <tkuratom@redhat.com>
+.. moduleauthor:: Ralph Bean <rbean@redhat.com>
 """
 
 import os
+import functools
 import logging
 import textwrap
 import warnings
 import requests
 
-from six.moves import urllib
 from distutils.version import LooseVersion
 
+# We unfortunately can't use python-six for this because there's an ancient
+# version on rhel7.  https://github.com/fedora-infra/python-fedora/issues/132
+try:
+    from urlparse import urlparse
+except ImportError:
+    # Python3 support
+    from urllib.parse import urlparse
+
 from fedora.client import OpenIdBaseClient, FedoraClientError, BaseClient, AuthError
 import fedora.client.openidproxyclient
 
@@ -40,7 +49,9 @@ __version__ = '2.0.0'
 log = logging.getLogger(__name__)
 
 BASE_URL = 'https://admin.fedoraproject.org/updates/'
+BODHI2_BASE_URL = 'https://bodhi.fedoraproject.org/'
 STG_BASE_URL = 'https://admin.stg.fedoraproject.org/updates/'
+BODHI2_STG_BASE_URL = 'https://bodhi.stg.fedoraproject.org/'
 STG_OPENID_API = 'https://id.stg.fedoraproject.org/api/v1/'
 
 
@@ -52,7 +63,6 @@ def BodhiClient(base_url=BASE_URL, staging=False, **kwargs):
     if staging:
         log.info('Using bodhi2 STAGING environment')
         base_url = STG_BASE_URL
-        fedora.client.openidproxyclient.FEDORA_OPENID_API = STG_OPENID_API
 
     log.debug('Querying bodhi API version')
     api_url = base_url + 'api_version'
@@ -64,7 +74,7 @@ def BodhiClient(base_url=BASE_URL, staging=False, **kwargs):
             data = data()
         server_version = LooseVersion(data['version'])
     except Exception as e:
-        if 'json' in str(type(e)).lower():
+        if 'json' in str(type(e)).lower() or 'json' in str(e).lower():
             # Claim that bodhi1 is on the server
             server_version = LooseVersion('0.9')
         else:
@@ -72,27 +82,46 @@ def BodhiClient(base_url=BASE_URL, staging=False, **kwargs):
 
     if server_version >= LooseVersion('2.0'):
         log.debug('Bodhi2 detected')
-        base_url = 'https://{}/'.format(urllib.parse.urlparse(response.url).netloc)
+        base_url = 'https://{}/'.format(urlparse(response.url).netloc)
         return Bodhi2Client(base_url=base_url, staging=staging, **kwargs)
     else:
         log.debug('Bodhi1 detected')
         return Bodhi1Client(base_url, **kwargs)
 
 
+def errorhandled(method):
+    """ A decorator for Bodhi2Client that raises exceptions on failure. """
+    @functools.wraps(method)
+    def wrapper(*args, **kwargs):
+        result = method(*args, **kwargs)
+
+        if 'errors' not in result:
+            return result
+
+        # Otherwise, there was a problem...
+        problems = 'An unhandled error occurred in the BodhiClient'
+        try:
+            problems = "\n".join([e['description'] for e in result['errors']])
+        except Exception:
+            pass
+        raise BodhiClientException(problems)
+    return wrapper
+
+
 class Bodhi2Client(OpenIdBaseClient):
 
-    def __init__(self, base_url=BASE_URL, username=None, password=None,
-                 staging=False, **kwargs):
+    def __init__(self, base_url=BODHI2_BASE_URL, username=None, password=None, staging=False, **kwargs):
+        if staging:
+            fedora.client.openidproxyclient.FEDORA_OPENID_API = STG_OPENID_API
+            base_url = BODHI2_STG_BASE_URL
+
         super(Bodhi2Client, self).__init__(base_url, login_url=base_url +
                 'login', username=username, **kwargs)
 
-        # bodhi1 client compatiblity
-        self.logged_in = False
         self.password = password
-        if username and password:
-            self.login(username, password)
-            self.logged_in = True
+        self.csrf_token = None
 
+    @errorhandled
     def save(self, **kwargs):
         """ Save an update.
 
@@ -136,11 +165,11 @@ class Bodhi2Client(OpenIdBaseClient):
         kwargs['csrf_token'] = self.csrf()
         if 'type_' in kwargs:
             # backwards compat
-            warnings.warn('Parameter "type_" is deprecated. Please use "type" instead.')
             kwargs['type'] = kwargs['type_']
         return self.send_request('updates/', verb='POST', auth=True,
                                  data=kwargs)
 
+    @errorhandled
     def request(self, update, request):
         """ Request an update state change.
 
@@ -153,12 +182,14 @@ class Bodhi2Client(OpenIdBaseClient):
                                  data={'update': update, 'request': request,
                                        'csrf_token': self.csrf()})
 
+    @errorhandled
     def delete(self, update):
         warnings.warn('Deleting updates has been disabled in Bodhi2. '
                       'This API call will unpush the update instead. '
                       'Please use `set_request(update, "unpush")` instead')
         self.request(update, 'unpush')
 
+    @errorhandled
     def query(self, **kwargs):
         """ Query bodhi for a list of updates.
 
@@ -172,7 +203,7 @@ class Bodhi2Client(OpenIdBaseClient):
             ``testing``, ``stable``, ``unpush``, ``obsolete`` or None.
         :kwarg mine: If True, only query the users updates.  Default: False.
         :kwarg packages: A space or comma delimited list of package names
-        :kwarg limit: The maximum number of updates to display.  Default: 10.
+        :kwarg limit: A deprecated argument, sets ``rows_per_page``. See its docstring for more info.
         :kwarg approved_since: A datetime string
         :kwarg builds: A space or comma delimited string of build nvrs
         :kwarg critpath: A boolean to query only critical path updates
@@ -191,7 +222,7 @@ class Bodhi2Client(OpenIdBaseClient):
             (``logout``, ``reboot``)
         :kwarg user: Query for updates submitted by a specific user.
         :kwarg rows_per_page: Limit the results to a certain number of rows per
-            page (min:1 max: 100)
+                    page (min:1 max: 100 default: 20)
         :kwarg page: Return a specific page of results
 
         """
@@ -202,6 +233,7 @@ class Bodhi2Client(OpenIdBaseClient):
             kwargs['user'] = self.username
         return self.send_request('updates', verb='GET', params=kwargs)
 
+    @errorhandled
     def comment(self, update, comment, karma=0, email=None):
         """ Add a comment to an update.
 
@@ -216,15 +248,16 @@ class Bodhi2Client(OpenIdBaseClient):
                       'karma': karma, 'email': email,
                       'csrf_token': self.csrf()})
 
+    @errorhandled
     def csrf(self, **kwargs):
-        # bodhi1 cli compatiblity for fedpkg update
-        if not self.logged_in:
+        if not self.csrf_token:
             if not self.password:
+                # bodhi1 cli compatiblity for fedpkg update
                 raise AuthError
             self.login(self.username, self.password)
-            self.logged_in = True
-        return self.send_request('csrf', verb='GET',
-                                 params=kwargs)['csrf_token']
+            self.csrf_token = self.send_request('csrf', verb='GET',
+                    params=kwargs)['csrf_token']
+        return self.csrf_token
 
     def parse_file(self, input_file):
         """ Parse an update template file.
@@ -268,6 +301,7 @@ class Bodhi2Client(OpenIdBaseClient):
 
         return updates
 
+    @errorhandled
     def latest_builds(self, package):
         return self.send_request('latest_builds', params={'package': package})
 
@@ -370,6 +404,7 @@ class Bodhi2Client(OpenIdBaseClient):
             val += "\n  %s\n" % ('%s%s' % (self.base_url, update['title']))
         return val
 
+    @errorhandled
     def get_releases(self, **kwargs):
         """ Return a list of bodhi releases.
 
